@@ -1066,7 +1066,6 @@ int fchmodat(int dir_fd, const char *path, mode_t mode, int flags) {
     return(r);
 
   st.st_mode=(mode&ALLPERMS)|(st.st_mode&~ALLPERMS);
-  INT_SEND_STAT(&st,chmod_func);
 
   /* see chmod() for comment */
   mode |= 0600;
@@ -1074,6 +1073,9 @@ int fchmodat(int dir_fd, const char *path, mode_t mode, int flags) {
     mode |= 0100;
 
   r=next_fchmodat(dir_fd, path, mode, flags);
+  if(!(r&&(errno==EOPNOTSUPP)))
+    INT_SEND_STAT(&st,chmod_func);
+
   if(r&&(errno==EPERM))
     r=0;
 #ifdef EFTYPE		/* available under FreeBSD kernel */
