@@ -37,8 +37,33 @@
 #endif
 #endif /* !defined(sun) && !defined(_NETINET_IN_H) */
 
+/* Endianness-agnostic swappers from byteswap.h */
+#define bswaps(x) \
+  ((uint16_t) ((((x) >> 8) & 0xff) | (((x) & 0xff) << 8)))
+#define bswapl(x) \
+  ((((x) & 0xff000000u) >> 24) | (((x) & 0x00ff0000u) >> 8) \
+   | (((x) & 0x0000ff00u) << 8) | (((x) & 0x000000ffu) << 24))
+#define bswapll(x) \
+  ((((x) & 0xff00000000000000ull) >> 56) \
+   | (((x) & 0x00ff000000000000ull) >> 40) \
+   | (((x) & 0x0000ff0000000000ull) >> 24) \
+   | (((x) & 0x000000ff00000000ull) >> 8) \
+   | (((x) & 0x00000000ff000000ull) << 8) \
+   | (((x) & 0x0000000000ff0000ull) << 24) \
+   | (((x) & 0x000000000000ff00ull) << 40) \
+   | (((x) & 0x00000000000000ffull) << 56))
+
 #define FAKEROOTKEY_ENV "FAKEROOTKEY"
 
+/* The magic value must be positive itself and be still positive if
+ * it becomes a high-dword of an uint64_t. Otherwise msgsnd(2) on x32
+ * platform thinks mtype is 64-bit negative number and returns -EINVAL.
+ * It should be also asymmetric it is possible to detect the endianness
+ * of message
+ */
+#define FAKEROOT_MAGIC_LE 0x78787878
+#define FAKEROOT_MAGIC_BE 0x75757575
+
 typedef uint32_t func_id_t;
 
 typedef uint64_t fake_ino_t;
@@ -83,12 +108,12 @@ struct fakexattr {
 #endif
 struct fake_msg {
 #ifndef FAKEROOT_FAKENET
-	long mtype; /* message type in SYSV message sending */
+	uint32_t magic; /* marker to detect cross-architecture mtype fluctuations */
 #endif
 	func_id_t       id; /* the requested function */
 #ifndef FAKEROOT_FAKENET
 	pid_t pid;
-	int serial;
+	int32_t serial;
 #endif
 	struct fakestat st;
 	struct fakexattr xattr;
@@ -98,4 +123,18 @@ struct fake_msg {
 #pragma pack()
 #endif
 
+#if __SUNPRO_C
+#pragma pack(4)
+#endif
+struct fake_msg_buf {
+#ifndef FAKEROOT_FAKENET
+	long mtype; /* message type in SYSV message sending */
+#endif
+	char msg[sizeof(struct fake_msg)];
+} FAKEROOT_ATTR(packed);
+
+#if __SUNPRO_C
+#pragma pack()
+#endif
+
 #endif
